@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
-    Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -29,8 +29,13 @@ USA.
 (standard-scheme-find-file-initialization
  '#(
     ("class"	(sos class))
+    ("geneqht"	(sos generic-procedure eqht))
+    ("generic"	(sos generic-procedure))
+    ("genmult"	(sos generic-procedure multiplexer))
     ("instance"	(sos instance))
     ("macros"	(sos macros))
     ("method"	(sos method))
     ("printer"	(sos printer))
-    ("slot"	(sos slot))))
\ No newline at end of file
+    ("recslot"	(sos record-slot-access))
+    ("slot"	(sos slot))
+    ("tvector"	(sos tagged-vector))))
\ No newline at end of file
