@@ -3,7 +3,7 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -93,12 +93,12 @@ USA.
        (string-every (char-set-predicate char-set:method-name) string)))
 
 (define char-set:method-name
-  (char-set-union (ascii-range->char-set (char->integer #\a)
-					 (fix:+ (char->integer #\z) 1))
-		  (ascii-range->char-set (char->integer #\A)
-					 (fix:+ (char->integer #\Z) 1))
-		  (ascii-range->char-set (char->integer #\0)
-					 (fix:+ (char->integer #\9) 1))
+  (char-set-union (ucs-range->char-set (char->integer #\a)
+				       (fix:+ (char->integer #\z) 1))
+		  (ucs-range->char-set (char->integer #\A)
+				       (fix:+ (char->integer #\Z) 1))
+		  (ucs-range->char-set (char->integer #\0)
+				       (fix:+ (char->integer #\9) 1))
 		  (char-set #\_ #\. #\: #\/)))
 
 (define (xml-rpc:parse-response document)
